/****************************************************************/
/* avltree.h - AVL Binary Search Tree Header			*/
/****************************************************************/
/*
 *  (c)2006,2008,2009 Todd Shadburn
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
/****************************************************************/


#ifndef _AVLTREE_H_
#define _AVLTREE_H_

#include <cutil/cutil_portable.h>


#define ESUCCESS		0
#define AVL_LEFT_HEAVY		1
#define AVL_BALANCED		0
#define AVL_RIGHT_HEAVY		-1
#define AVL_NODE_HIDDEN		1


#define AVLTREE_ITER_ASCEND		1
#define AVLTREE_ITER_DESCEND		2
#define AVLTREE_ITER_F_COMPLETE		1


/* node navigation macros: Please use the functions for tree navigation
 * unless you really know what you're doing. Using these directly could
 * cause your application to break with future releases of this library.
*/
#define avltree_root(tree)	((tree)->root)
#define avltree_size(tree)	((tree)->size)
#define avlnode_left(node)	((node)->left)
#define avlnode_right(node)	((node)->right)
#define avlnode_data(node)	((node)->data)
#define avlnode_is_eob(node)	((node) == NULL)
#define avlnode_is_leaf(node)	((node)->left == NULL && (node)->right == NULL)


typedef int (*AVLNodeCompare)(const void *key1, const void *key2);
typedef int (*AVLNodeDestroy)(void *data);


typedef struct _AVLNode	AVLNode;
struct _AVLNode {
	AVLNode		*left;
	AVLNode		*right;
	void		*data;
	int		hidden;
	int		factor;
};


typedef struct _AVLTree		AVLTree;
struct _AVLTree {
	unsigned long	size;
	AVLNode		*root;
	AVLNodeCompare	compare;
	AVLNodeDestroy	destroy;
};


/* Opaque iterator structure */
typedef struct _avltree_iter_t avltree_iter_t;


/* Basic API functions */
CUTIL_API int avltree_new (AVLTree **treep,
		AVLNodeCompare compare,
		AVLNodeDestroy destroy);
CUTIL_API int avltree_free (AVLTree ** treep);
CUTIL_API int avltree_insert (AVLTree *tree, void *data);
CUTIL_API int avltree_remove (AVLTree *tree, const void *data);
CUTIL_API void *avltree_find (AVLTree *tree, void *data);
CUTIL_API int avltree_exists (AVLTree *tree, void *data);

CUTIL_API void *avltree_find_by_compare (AVLTree *tree,
				AVLNodeCompare compare,
				void *data);
CUTIL_API int avltree_exists_by_compare (AVLTree *tree,
				AVLNodeCompare compare,
				void *data);

avltree_iter_t *avltree_iter_new(AVLTree *tree, int direction);
int avltree_iter_free(avltree_iter_t *iter);
void *avltree_iter_next(avltree_iter_t *iter);


/* Extended API functions */
CUTIL_API int avltree_insert_left (AVLTree *tree, AVLNode *node, void *data);
CUTIL_API int avltree_insert_right (AVLTree *tree, AVLNode *node, void *data);
CUTIL_API int avltree_remove_left (AVLTree *tree, AVLNode *node);
CUTIL_API int avltree_remove_right (AVLTree *tree, AVLNode *node);

/*
CUTIL_API int avltree_size (AVLTree *tree);
CUTIL_API AVLNode *avltree_root (AVLTree *tree);
CUTIL_API int avlnode_is_leaf (AVLNode *node);
CUTIL_API AVLNode *avlnode_left (AVLNode *node);
CUTIL_API AVLNode *avlnode_right (AVLNode *node);
CUTIL_API void *avlnode_data (AVLNode *node);
*/

#endif


